#include "model.h"
#include "mesh.h"
#include "image.h"
#include "texture.h"
#include "phongmaterial.h"

Model::~Model()
{
    destroy();
    destroyGL();
    for(Mesh *m : m_meshes)
        delete m;
}

Image* Model::getImage(const std::string &name)
{
    for(TextureImg &tex : m_textures)
    {
        if(name.compare(tex.name) == 0)
            return tex.img;
    }
    return NULL;
}

Texture* Model::getTexture(const std::string &name)
{
    for(TextureImg &tex : m_textures)
    {
        if(name.compare(tex.name) == 0)
            return tex.tex;
    }
    return NULL;
}

void Model::destroy()
{
    for(Mesh* m : m_meshes)
        m->clearData();
    for(TextureImg &tex : m_textures)
    {
        delete tex.img;
        tex.img = NULL;
    }
}

void Model::initGL()
{
    for(Mesh* m : m_meshes)
        m->initGL();
    for(TextureImg &tex : m_textures)
    {
        if(tex.tex == NULL)
            tex.tex = new Texture(tex.img);
    }
}

void Model::destroyGL()
{
    for(Mesh* m : m_meshes)
        m->destroyGL();
    for(TextureImg &tex : m_textures)
    {
        if(tex.tex != NULL)
        {
            delete tex.tex;
            tex.tex = NULL;
        }
    }
}

bool Model::save(const std::string &filename, const std::string &texturesPath)
{
    // open file
    std::FILE *file = std::fopen(filename.c_str(), "w");
    if(file == NULL)
        return false;
    
    bool ok = false;
    int size = m_meshes.size();
    if(std::fwrite(&size, sizeof(int), 1, file))
    {
        ok = true;
        for(Mesh* m : m_meshes)
        {
            ok = ok && m->serialize(file);
            PhongMaterial* mat = (PhongMaterial*)m->getMaterial();
            ok = ok && mat->serialize(file);
        }
        for(Model::TextureImg &texImg : m_textures)
        {
            std::string texFilename = texturesPath + texImg.name + ".png";
            if(texImg.img == NULL && texImg.tex != NULL)
                texImg.img = texImg.tex->getData();
            if(texImg.img != NULL)
                ok = ok && texImg.img->save(texFilename);
        }
        std::fclose(file);
    }
    
    // finishing
    return ok;
}

Model* Model::load(const std::string &filename, const std::string &texturesPath)
{
    // open file
    std::FILE *file = std::fopen(filename.c_str(), "r");
    if(file == NULL)
        return NULL;
    
    bool ok = false;
    
    
    // finishing
    return NULL;
}
